@@ -377,11 +377,17 @@ setReplaceMethod("names", "Seqinfo",
 )
 
 setReplaceMethod("seqlevels", "Seqinfo",
-    function(x, force=FALSE, value)
+    function(x, force=FALSE,
+             pruning.mode=c("error", "coarse", "fine", "tidy"),
+             value)
     {
         if (!identical(force, FALSE))
             warning("'force' is ignored in \"seqlevels<-\" method ",
                     "for Seqinfo objects")
+        pruning.mode <- match.arg(pruning.mode)
+        if (pruning.mode != "error")
+            warning("'pruning.mode' is ignored in \"seqlevels<-\" method ",
+                    "for Seqinfo objects")
         new2old <- getSeqlevelsReplacementMode(value, seqlevels(x))
         if (identical(new2old, -3L)) {
             ## "renaming" mode
